@@ -30,8 +30,9 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-"""
-Library for retrieving information about catkin packages.
-"""
+"""Library for retrieving information about catkin packages."""
 
-__version__ = '0.3.9'  # same version as in setup.py
+# same version as in:
+# - setup.py
+# - stdeb.cfg
+__version__ = '0.4.10'
