@@ -31,9 +31,7 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-'''
-Extract log information from repositories.
-'''
+"""Extract log information from repositories."""
 
 import os
 import re
@@ -128,7 +126,7 @@ class VcsClientBase(object):
 
 class GitClient(VcsClientBase):
 
-    type = 'git'
+    type = 'git'  # noqa: A003
 
     def __init__(self, path):
         super(GitClient, self).__init__(path)
@@ -139,7 +137,7 @@ class GitClient(VcsClientBase):
 
     # query author
     def _get_author(self, hash_):
-        cmd = [self._executable, 'log', hash_, '-n', '1', '--format=format:%an']
+        cmd = [self._executable, 'log', hash_, '-n', '1', '--format=format:%aN']
         result = self._run_command(cmd)
         if result['returncode']:
             raise RuntimeError('Could not fetch author:\n%s' % result['output'])
@@ -257,7 +255,7 @@ class GitClient(VcsClientBase):
 
 class HgClient(VcsClientBase):
 
-    type = 'hg'
+    type = 'hg'  # noqa: A003
 
     def __init__(self, path):
         super(HgClient, self).__init__(path)
@@ -289,11 +287,13 @@ class HgClient(VcsClientBase):
         return tags
 
     def get_latest_tag_name(self):
-        cmd_log = [self._executable, 'log', '--rev', '.', '--template', '{latesttagdistance}']
+        cmd_log = [self._executable, 'log', '--rev', '.', '--template', '{latesttag}']
         result_log = self._run_command(cmd_log)
         if result_log['returncode']:
             raise RuntimeError('Could not fetch latest tag:\n%s' % result_log['output'])
         tag_name = result_log['output']
+        if tag_name == 'null':
+            raise RuntimeError('Could not find latest tagn')
         return tag_name
 
     def get_log_entries(self, from_tag, to_tag, skip_merges=False):
