@@ -30,10 +30,7 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-"""
-Library for parsing package.xml and providing an object
-representation.
-"""
+"""Library for parsing package.xml and providing an object representation."""
 
 from __future__ import print_function
 
@@ -43,18 +40,19 @@ import re
 import sys
 import xml.dom.minidom as dom
 
+from catkin_pkg.condition import evaluate_condition
+
 PACKAGE_MANIFEST_FILENAME = 'package.xml'
 
 
 class Package(object):
-    """
-    Object representation of a package manifest file
-    """
+    """Object representation of a package manifest file."""
+
     __slots__ = [
         'package_format',
         'name',
         'version',
-        'version_abi',
+        'version_compatibility',
         'description',
         'maintainers',
         'licenses',
@@ -69,12 +67,16 @@ class Package(object):
         'doc_depends',
         'conflicts',
         'replaces',
+        'group_depends',
+        'member_of_groups',
         'exports',
         'filename'
     ]
 
     def __init__(self, filename=None, **kwargs):
         """
+        Initialize Package.
+
         :param filename: location of package.xml.  Necessary if
           converting ``${prefix}`` in ``<export>`` values, ``str``.
         """
@@ -99,6 +101,7 @@ class Package(object):
                         slot.append(deepcopy(d))
             del kwargs['run_depends']
         self.filename = filename
+        self.licenses = [l if isinstance(l, License) else License(l) for l in self.licenses]
         # verify that no unknown keywords are passed
         unknown = set(kwargs.keys()).difference(self.__slots__)
         if unknown:
@@ -130,7 +133,7 @@ class Package(object):
 
     def has_buildtool_depend_on_catkin(self):
         """
-        Returns True if this Package buildtool depends on catkin, otherwise False
+        Return True if this Package buildtool depends on catkin, otherwise False.
 
         :returns: True if the given package buildtool depends on catkin
         :rtype: bool
@@ -145,16 +148,20 @@ class Package(object):
         :rtype: str
         :raises: :exc:`InvalidPackage`
         """
-        build_type_exports = [e.content for e in self.exports if e.tagname == 'build_type']
+        # for backward compatibility a build type without an evaluated
+        # condition is still being considered (i.e. evaluated_condition is None)
+        build_type_exports = [
+            e.content for e in self.exports
+            if e.tagname == 'build_type' and e.evaluated_condition is not False]
         if not build_type_exports:
             return 'catkin'
         if len(build_type_exports) == 1:
             return build_type_exports[0]
-        raise InvalidPackage('Only one <build_type> element is permitted.')
+        raise InvalidPackage('Only one <build_type> element is permitted.', self.filename)
 
     def has_invalid_metapackage_dependencies(self):
         """
-        Returns True if this package has invalid dependencies for a metapackage
+        Return True if this package has invalid dependencies for a metapackage.
 
         This is defined by REP-0127 as any non-run_depends dependencies other then a buildtool_depend on catkin.
 
@@ -166,16 +173,43 @@ class Package(object):
 
     def is_metapackage(self):
         """
-        Returns True if this pacakge is a metapackage, otherwise False
+        Return True if this pacakge is a metapackage, otherwise False.
 
         :returns: True if metapackage, else False
         :rtype: bool
         """
         return 'metapackage' in [e.tagname for e in self.exports]
 
+    def evaluate_conditions(self, context):
+        """
+        Evaluate the conditions of all dependencies and memberships.
+
+        :param context: A dictionary with key value pairs to replace variables
+          starting with $ in the condition.
+        :raises: :exc:`ValueError` if any condition fails to parse
+        """
+        for attr in (
+            'build_depends',
+            'buildtool_depends',
+            'build_export_depends',
+            'buildtool_export_depends',
+            'exec_depends',
+            'test_depends',
+            'doc_depends',
+            'conflicts',
+            'replaces',
+            'group_depends',
+            'member_of_groups',
+            'exports',
+        ):
+            conditionals = getattr(self, attr)
+            for conditional in conditionals:
+                conditional.evaluate_condition(context)
+
     def validate(self, warnings=None):
         """
-        makes sure all standards for packages are met
+        Make sure all standards for packages are met.
+
         :param package: Package to check
         :param warnings: Print warnings if None or return them in the given list
         :raises InvalidPackage: in case validation fails
@@ -193,34 +227,34 @@ class Package(object):
         if not re.match('^[a-zA-Z0-9][a-zA-Z0-9_-]*$', self.name):
             errors.append('Package name "%s" does not follow naming conventions' % self.name)
         else:
-            if self.get_build_type() == 'catkin':
-                if not re.match('^[a-z][a-z0-9_]*$', self.name):
-                    new_warnings.append(
-                        'Catkin package name "%s" does not follow the naming conventions. It should start with '
-                        'a lower case letter and only contain lower case letters, digits, and underscores.' % self.name)
-            else:
-                if not re.match('^[a-z][a-z0-9_-]*$', self.name):
-                    new_warnings.append(
-                        'Non-catkin package name "%s" does not follow the naming conventions. It should start with'
-                        'a lower case letter and only contain lower case letters, digits, underscores, and dashes.' % self.name)
+            if not re.match('^[a-z][a-z0-9_-]*$', self.name):
+                new_warnings.append(
+                    'Package name "%s" does not follow the naming conventions. It should start with '
+                    'a lower case letter and only contain lower case letters, digits, underscores, and dashes.' % self.name)
 
+        version_regexp = '^[0-9]+\.[0-9]+\.[0-9]+$'
         if not self.version:
             errors.append('Package version must not be empty')
-        elif not re.match('^[0-9]+\.[0-9]+\.[0-9]+$', self.version):
+        elif not re.match(version_regexp, self.version):
             errors.append('Package version "%s" does not follow version conventions' % self.version)
         elif not re.match('^(0|[1-9][0-9]*)\.(0|[1-9][0-9]*)\.(0|[1-9][0-9]*)$', self.version):
             new_warnings.append('Package "%s" does not follow the version conventions. It should not contain leading zeros (unless the number is 0).' % self.name)
+        if self.version_compatibility:
+            if not re.match(version_regexp, self.version_compatibility):
+                errors.append(
+                    "Package compatibility version '%s' does not follow "
+                    'version conventions' % self.version_compatibility)
 
         if not self.description:
             errors.append('Package description must not be empty')
 
         if not self.maintainers:
-            errors.append("Package '{}' must declare at least one maintainer".format(self.name))
+            errors.append("Package '{0}' must declare at least one maintainer".format(self.name))
         for maintainer in self.maintainers:
             try:
                 maintainer.validate()
             except InvalidPackage as e:
-                errors.append(str(e))
+                errors.append(e.msg)
             if not maintainer.email:
                 errors.append('Maintainers must have an email address')
 
@@ -234,7 +268,7 @@ class Package(object):
                 try:
                     author.validate()
                 except InvalidPackage as e:
-                    errors.append(str(e))
+                    errors.append(e.msg)
 
         dep_types = {
             'build': self.build_depends,
@@ -248,7 +282,15 @@ class Package(object):
         for dep_type, depends in dep_types.items():
             for depend in depends:
                 if depend.name == self.name:
-                    errors.append('The package must not "%s_depend" on a package with the same name as this package' % dep_type)
+                    errors.append('The package "%s" must not "%s_depend" on a package with the same name as this package' % (self.name, dep_type))
+
+        if (
+            set([d.name for d in self.group_depends]) &
+            set([g.name for g in self.member_of_groups])
+        ):
+            errors.append(
+                "The package must not 'group_depend' on a package which it "
+                'also declares to be a member of')
 
         if self.is_metapackage():
             if not self.has_buildtool_depend_on_catkin():
@@ -256,7 +298,8 @@ class Package(object):
                 new_warnings.append('Metapackage "%s" must buildtool_depend on catkin.' % self.name)
             if self.has_invalid_metapackage_dependencies():
                 new_warnings.append('Metapackage "%s" should not have other dependencies besides a '
-                                    'buildtool_depend on catkin and run_depends.' % self.name)
+                                    'buildtool_depend on catkin and %s.' %
+                                    (self.name, 'run_depends' if self.package_format == 1 else 'exec_depends'))
 
         for warning in new_warnings:
             if warnings is None:
@@ -265,13 +308,19 @@ class Package(object):
                 warnings.append(warning)
 
         if errors:
-            raise InvalidPackage('\n'.join(errors))
+            raise InvalidPackage('\n'.join(errors), self.filename)
 
 
 class Dependency(object):
-    __slots__ = ['name', 'version_lt', 'version_lte', 'version_eq', 'version_gte', 'version_gt']
+    __slots__ = [
+        'name',
+        'version_lt', 'version_lte', 'version_eq', 'version_gte', 'version_gt',
+        'condition',
+        'evaluated_condition',
+    ]
 
     def __init__(self, name, **kwargs):
+        self.evaluated_condition = None
         for attr in self.__slots__:
             value = kwargs[attr] if attr in kwargs else None
             setattr(self, attr, value)
@@ -284,22 +333,38 @@ class Dependency(object):
     def __eq__(self, other):
         if not isinstance(other, Dependency):
             return False
-        return all([getattr(self, attr) == getattr(other, attr) for attr in self.__slots__])
+        return all(getattr(self, attr) == getattr(other, attr) for attr in self.__slots__)
 
     def __hash__(self):
-        return hash(tuple([getattr(self, slot) for slot in self.__slots__]))
+        return hash(tuple(getattr(self, slot) for slot in self.__slots__))
 
     def __str__(self):
         return self.name
 
+    def evaluate_condition(self, context):
+        """
+        Evaluate the condition.
+
+        The result is also stored in the member variable `evaluated_condition`.
+
+        :param context: A dictionary with key value pairs to replace variables
+          starting with $ in the condition.
+
+        :returns: True if the condition evaluates to True, else False
+        :raises: :exc:`ValueError` if the condition fails to parse
+        """
+        self.evaluated_condition = evaluate_condition(self.condition, context)
+        return self.evaluated_condition
+
 
 class Export(object):
-    __slots__ = ['tagname', 'attributes', 'content']
+    __slots__ = ['tagname', 'attributes', 'content', 'evaluated_condition']
 
     def __init__(self, tagname, content=None):
         self.tagname = tagname
         self.attributes = {}
         self.content = content
+        self.evaluated_condition = None
 
     def __str__(self):
         txt = '<%s' % self.tagname
@@ -311,6 +376,30 @@ class Export(object):
             txt += '/>'
         return txt
 
+    def evaluate_condition(self, context):
+        """
+        Evaluate the condition.
+
+        The result is also stored in the member variable `evaluated_condition`.
+
+        :param context: A dictionary with key value pairs to replace variables
+          starting with $ in the condition.
+
+        :returns: True if the condition evaluates to True, else False
+        :raises: :exc:`ValueError` if the condition fails to parse
+        """
+        self.evaluated_condition = evaluate_condition(self.attributes.get('condition'), context)
+        return self.evaluated_condition
+
+
+# Subclassing ``str`` to keep backward compatibility.
+class License(str):
+
+    def __new__(cls, value, file_=None):
+        obj = str.__new__(cls, str(value))
+        obj.file = file_
+        return obj
+
 
 class Person(object):
     __slots__ = ['name', 'email']
@@ -347,7 +436,8 @@ class Url(object):
 
 
 def parse_package_for_distutils(path=None):
-    print('WARNING: %s/setup.py: catkin_pkg.package.parse_package_for_distutils() is deprecated. Please use catkin_pkg.python_setup.generate_distutils_setup(**kwargs) instead.' % os.path.basename(os.path.abspath('.')))
+    print('WARNING: %s/setup.py: catkin_pkg.package.parse_package_for_distutils() is deprecated. Please use catkin_pkg.python_setup.generate_distutils_setup(**kwargs) instead.' %
+          os.path.basename(os.path.abspath('.')))
     from .python_setup import generate_distutils_setup
     data = {}
     if path is not None:
@@ -356,12 +446,20 @@ def parse_package_for_distutils(path=None):
 
 
 class InvalidPackage(Exception):
-    pass
+
+    def __init__(self, msg, package_path=None):
+        self.msg = msg
+        self.package_path = package_path
+        Exception.__init__(self, self.msg)
+
+    def __str__(self):
+        result = '' if not self.package_path else "Error(s) in package '%s':\n" % self.package_path
+        return result + Exception.__str__(self)
 
 
 def package_exists_at(path):
     """
-    Checks that a package exists at the given path
+    Check that a package exists at the given path.
 
     :param path: path to a package
     :type path: str
@@ -399,6 +497,7 @@ def _get_package_xml(path):
     with open(filename, 'r', **kwargs) as f:
         return f.read(), filename
 
+
 def parse_package(path, warnings=None):
     """
     Parse package manifest.
@@ -412,11 +511,7 @@ def parse_package(path, warnings=None):
     :raises: :exc:`IOError`
     """
     xml, filename = _get_package_xml(path)
-    try:
-        return parse_package_string(xml, filename, warnings=warnings)
-    except InvalidPackage as e:
-        e.args = ['Invalid package manifest "%s": %s' % (filename, e.message)]
-        raise
+    return parse_package_string(xml, filename, warnings=warnings)
 
 
 def _check_known_attributes(node, known):
@@ -428,6 +523,7 @@ def _check_known_attributes(node, known):
             return ['The "%s" tag must not have the following attributes: %s' % (node.tagName, ', '.join(unknown_attrs))]
     return []
 
+
 def parse_package_string(data, filename=None, warnings=None):
     """
     Parse package.xml string contents.
@@ -441,31 +537,34 @@ def parse_package_string(data, filename=None, warnings=None):
     try:
         root = dom.parseString(data)
     except Exception as ex:
-        raise InvalidPackage('The manifest contains invalid XML:\n%s' % ex)
+        raise InvalidPackage('The manifest contains invalid XML:\n%s' % ex, filename)
 
     pkg = Package(filename)
 
     # verify unique root node
     nodes = _get_nodes(root, 'package')
     if len(nodes) != 1:
-        raise InvalidPackage('The manifest must contain a single "package" root tag')
+        raise InvalidPackage('The manifest must contain a single "package" root tag', filename)
     root = nodes[0]
 
     # format attribute
     value = _get_node_attr(root, 'format', default=1)
     pkg.package_format = int(value)
-    assert pkg.package_format in [1, 2], "Unable to handle package.xml format version '%d', please update catkin_pkg (e.g. on Ubuntu/Debian use: sudo apt-get update && sudo apt-get install --only-upgrade python-catkin-pkg)" % pkg.package_format
+    assert pkg.package_format in (1, 2, 3), \
+        "Unable to handle package.xml format version '%d', please update catkin_pkg " \
+        '(e.g. on Ubuntu/Debian use: sudo apt-get update && sudo apt-get install --only-upgrade python-catkin-pkg)' % pkg.package_format
 
     # name
-    pkg.name = _get_node_value(_get_node(root, 'name'))
+    pkg.name = _get_node_value(_get_node(root, 'name', filename))
 
-    # version and optional abi
-    version_node = _get_node(root, 'version')
+    # version and optional compatibility
+    version_node = _get_node(root, 'version', filename)
     pkg.version = _get_node_value(version_node)
-    pkg.version_abi = _get_node_attr(version_node, 'abi', default=None)
+    pkg.version_compatibility = _get_node_attr(
+        version_node, 'compatibility', default=None)
 
     # description
-    pkg.description = _get_node_value(_get_node(root, 'description'), allow_xml=True, apply_str=False)
+    pkg.description = _get_node_value(_get_node(root, 'description', filename), allow_xml=True, apply_str=False)
 
     # at least one maintainer, all must have email
     maintainers = _get_nodes(root, 'maintainer')
@@ -494,7 +593,10 @@ def parse_package_string(data, filename=None, warnings=None):
     # at least one license
     licenses = _get_nodes(root, 'license')
     for node in licenses:
-        pkg.licenses.append(_get_node_value(node))
+        pkg.licenses.append(License(
+            _get_node_value(node),
+            _get_node_attr(node, 'file', default=None)
+        ))
 
     errors = []
     # dependencies and relationships
@@ -505,7 +607,7 @@ def parse_package_string(data, filename=None, warnings=None):
         for d in run_depends:
             pkg.build_export_depends.append(deepcopy(d))
             pkg.exec_depends.append(deepcopy(d))
-    if pkg.package_format == 2:
+    if pkg.package_format != 1:
         pkg.build_export_depends = _get_dependencies(root, 'build_export_depend')
         pkg.buildtool_export_depends = _get_dependencies(root, 'buildtool_export_depend')
         pkg.exec_depends = _get_dependencies(root, 'exec_depend')
@@ -529,6 +631,10 @@ def parse_package_string(data, filename=None, warnings=None):
     pkg.conflicts = _get_dependencies(root, 'conflict')
     pkg.replaces = _get_dependencies(root, 'replace')
 
+    # group dependencies and memberships
+    pkg.group_depends = _get_group_dependencies(root, 'group_depend')
+    pkg.member_of_groups = _get_group_memberships(root, 'member_of_group')
+
     if pkg.package_format == 1:
         for test_depend in pkg.test_depends:
             same_build_depends = ['build_depend' for d in pkg.build_depends if d.name == test_depend.name]
@@ -537,7 +643,7 @@ def parse_package_string(data, filename=None, warnings=None):
                 errors.append('The test dependency on "%s" is redundant with: %s' % (test_depend.name, ', '.join(same_build_depends + same_run_depends)))
 
     # exports
-    export_node = _get_optional_node(root, 'export')
+    export_node = _get_optional_node(root, 'export', filename)
     if export_node is not None:
         exports = []
         for node in [n for n in export_node.childNodes if n.nodeType == n.ELEMENT_NODE]:
@@ -550,9 +656,11 @@ def parse_package_string(data, filename=None, warnings=None):
     # verify that no unsupported tags and attributes are present
     errors += _check_known_attributes(root, ['format'])
     depend_attributes = ['version_lt', 'version_lte', 'version_eq', 'version_gte', 'version_gt']
+    if pkg.package_format > 2:
+        depend_attributes.append('condition')
     known = {
         'name': [],
-        'version': ['abi'],
+        'version': ['compatibility'],
         'description': [],
         'maintainer': ['email'],
         'license': [],
@@ -569,7 +677,7 @@ def parse_package_string(data, filename=None, warnings=None):
         known.update({
             'run_depend': depend_attributes,
         })
-    if pkg.package_format == 2:
+    if pkg.package_format != 1:
         known.update({
             'build_export_depend': depend_attributes,
             'buildtool_export_depend': depend_attributes,
@@ -577,6 +685,15 @@ def parse_package_string(data, filename=None, warnings=None):
             'exec_depend': depend_attributes,
             'doc_depend': depend_attributes,
         })
+    if pkg.package_format > 2:
+        known.update({
+            'group_depend': ['condition'],
+            'member_of_group': ['condition']
+        })
+    if pkg.package_format > 2:
+        known.update({
+            'license': ['file'],
+        })
     nodes = [n for n in root.childNodes if n.nodeType == n.ELEMENT_NODE]
     unknown_tags = set([n.tagName for n in nodes if n.tagName not in known.keys()])
     if unknown_tags:
@@ -589,7 +706,7 @@ def parse_package_string(data, filename=None, warnings=None):
                 errors.append('The "%s" tag must not contain the following children: %s' % (node.tagName, ', '.join([n.tagName for n in subnodes])))
 
     if errors:
-        raise InvalidPackage('Error(s) in %s:%s' % (filename, ''.join(['\n- %s' % e for e in errors])))
+        raise InvalidPackage('Error(s):%s' % (''.join(['\n- %s' % e for e in errors])), filename)
 
     pkg.validate(warnings=warnings)
 
@@ -600,17 +717,17 @@ def _get_nodes(parent, tagname):
     return [n for n in parent.childNodes if n.nodeType == n.ELEMENT_NODE and n.tagName == tagname]
 
 
-def _get_node(parent, tagname):
+def _get_node(parent, tagname, filename):
     nodes = _get_nodes(parent, tagname)
     if len(nodes) != 1:
-        raise InvalidPackage('The manifest must contain exactly one "%s" tags' % tagname)
+        raise InvalidPackage('The manifest must contain exactly one "%s" tag' % tagname, filename)
     return nodes[0]
 
 
-def _get_optional_node(parent, tagname):
+def _get_optional_node(parent, tagname, filename):
     nodes = _get_nodes(parent, tagname)
     if len(nodes) > 1:
-        raise InvalidPackage('The manifest must not contain more than one "%s" tags' % tagname)
+        raise InvalidPackage('The manifest must not contain more than one "%s" tags' % tagname, filename)
     return nodes[0] if nodes else None
 
 
@@ -624,17 +741,8 @@ def _get_node_value(node, allow_xml=False, apply_str=True):
     return value
 
 
-def _get_optional_node_value(parent, tagname, default=None):
-    node = _get_optional_node(parent, tagname)
-    if node is None:
-        return default
-    return _get_node_value(node)
-
-
 def _get_node_attr(node, attr, default=False):
-    """
-    :param default: False means value is required
-    """
+    """:param default: False means value is required."""
     if node.hasAttribute(attr):
         return str(node.getAttribute(attr))
     if default is False:
@@ -646,7 +754,29 @@ def _get_dependencies(parent, tagname):
     depends = []
     for node in _get_nodes(parent, tagname):
         depend = Dependency(_get_node_value(node))
-        for attr in ['version_lt', 'version_lte', 'version_eq', 'version_gte', 'version_gt']:
+        for attr in ('version_lt', 'version_lte', 'version_eq', 'version_gte', 'version_gt', 'condition'):
             setattr(depend, attr, _get_node_attr(node, attr, None))
         depends.append(depend)
     return depends
+
+
+def _get_group_dependencies(parent, tagname):
+    from .group_dependency import GroupDependency
+    depends = []
+    for node in _get_nodes(parent, tagname):
+        depends.append(
+            GroupDependency(
+                _get_node_value(node),
+                condition=_get_node_attr(node, 'condition', default=None)))
+    return depends
+
+
+def _get_group_memberships(parent, tagname):
+    from .group_membership import GroupMembership
+    memberships = []
+    for node in _get_nodes(parent, tagname):
+        memberships.append(
+            GroupMembership(
+                _get_node_value(node),
+                condition=_get_node_attr(node, 'condition', default=None)))
+    return memberships
