@@ -30,9 +30,7 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-"""
-Library to find packages in the filesystem.
-"""
+"""Library to find packages in the filesystem."""
 
 import multiprocessing
 import os
@@ -46,7 +44,10 @@ def find_package_paths(basepath, exclude_paths=None, exclude_subspaces=False):
     """
     Crawls the filesystem to find package manifest files.
 
-    When a subfolder contains a file ``CATKIN_IGNORE`` it is ignored.
+    When a subfolder contains either of the following files it is ignored:
+    - ``AMENT_IGNORE``
+    - ``CATKIN_IGNORE``
+    - ``COLCON_IGNORE``
 
     :param basepath: The path to search in, ``str``
     :param exclude_paths: A list of paths which should not be searched, ``list``
@@ -57,7 +58,9 @@ def find_package_paths(basepath, exclude_paths=None, exclude_subspaces=False):
     paths = []
     real_exclude_paths = [os.path.realpath(p) for p in exclude_paths] if exclude_paths is not None else []
     for dirpath, dirnames, filenames in os.walk(basepath, followlinks=True):
-        if 'CATKIN_IGNORE' in filenames or \
+        if 'AMENT_IGNORE' in filenames or \
+            'CATKIN_IGNORE' in filenames or \
+            'COLCON_IGNORE' in filenames or \
             os.path.realpath(dirpath) in real_exclude_paths or \
                 (exclude_subspaces and '.catkin' in filenames):
             del dirnames[:]
@@ -87,7 +90,7 @@ def find_packages(basepath, exclude_paths=None, exclude_subspaces=False, warning
     package_paths_by_name = {}
     for path, package in packages.items():
         if package.name not in package_paths_by_name:
-            package_paths_by_name[package.name] = set([])
+            package_paths_by_name[package.name] = set()
         package_paths_by_name[package.name].add(path)
     duplicates = dict([(name, paths) for name, paths in package_paths_by_name.items() if len(paths) > 1])
     if duplicates:
@@ -97,6 +100,7 @@ def find_packages(basepath, exclude_paths=None, exclude_subspaces=False, warning
 
 
 class _PackageParser(object):
+
     def __init__(self, capture_warnings):
         self.capture_warnings = capture_warnings
 
@@ -161,7 +165,7 @@ def find_packages_allowing_duplicates(basepath, exclude_paths=None, exclude_subs
 
 def verify_equal_package_versions(packages):
     """
-    Verifies that all packages have the same version number.
+    Verify that all packages have the same version number.
 
     :param packages: The list of ``Package`` objects, ``list``
     :returns: The version number
