@@ -1,15 +1,28 @@
-import unittest
-
+import os.path
 # Redirect stderr to stdout to suppress output in tests
 import sys
-sys.stderr = sys.stdout
-
-from catkin_pkg.package import Dependency, Export, InvalidPackage, Package, Person, _check_known_attributes
+import unittest
 
 import xml.dom.minidom as dom
 
+from catkin_pkg.package import (
+    _check_known_attributes,
+    Dependency,
+    Export,
+    InvalidPackage,
+    License,
+    Package,
+    parse_package,
+    Person,
+)
+
 from mock import Mock
 
+sys.stderr = sys.stdout
+
+test_data_dir = os.path.join(os.path.dirname(__file__), 'data', 'package')
+
+
 class PackageTest(unittest.TestCase):
 
     def get_maintainer(self):
@@ -18,6 +31,11 @@ class PackageTest(unittest.TestCase):
         maint.name = 'John Foo'
         return maint
 
+    def get_group_dependency(self, name):
+        group = Mock()
+        group.name = name
+        return group
+
     def test_init(self):
         maint = self.get_maintainer()
         pack = Package(name='foo',
@@ -25,10 +43,13 @@ class PackageTest(unittest.TestCase):
                        maintainers=[maint],
                        licenses=['BSD'])
         self.assertEqual(None, pack.filename)
+        self.assertEqual('0.0.0', pack.version)
+        self.assertEqual(None, pack.version_compatibility)
         self.assertEqual([], pack.urls)
         self.assertEqual([], pack.authors)
         self.assertEqual([maint], pack.maintainers)
         self.assertEqual(['BSD'], pack.licenses)
+        self.assertEqual([None], [l.file for l in pack.licenses])
         self.assertEqual([], pack.build_depends)
         self.assertEqual([], pack.buildtool_depends)
         self.assertEqual([], pack.run_depends)
@@ -36,6 +57,8 @@ class PackageTest(unittest.TestCase):
         self.assertEqual([], pack.conflicts)
         self.assertEqual([], pack.replaces)
         self.assertEqual([], pack.exports)
+        self.assertEqual([], pack.group_depends)
+        self.assertEqual([], pack.member_of_groups)
         pack = Package('foo',
                        name='bar',
                        version='0.0.0',
@@ -51,13 +74,18 @@ class PackageTest(unittest.TestCase):
                          version_lte=2,
                          version_eq=3,
                          version_gte=4,
-                         version_gt=5)
-        self.assertEquals('foo', dep.name)
-        self.assertEquals(1, dep.version_lt)
-        self.assertEquals(2, dep.version_lte)
-        self.assertEquals(3, dep.version_eq)
-        self.assertEquals(4, dep.version_gte)
-        self.assertEquals(5, dep.version_gt)
+                         version_gt=5,
+                         condition='$foo == 23 and $bar != 42')
+        self.assertEqual('foo', dep.name)
+        self.assertEqual(1, dep.version_lt)
+        self.assertEqual(2, dep.version_lte)
+        self.assertEqual(3, dep.version_eq)
+        self.assertEqual(4, dep.version_gte)
+        self.assertEqual(5, dep.version_gt)
+        self.assertFalse(dep.evaluate_condition({'foo': 23, 'bar': 42}))
+        self.assertFalse(dep.evaluated_condition)
+        self.assertTrue(dep.evaluate_condition({'foo': 23, 'bar': 43}))
+        self.assertTrue(dep.evaluated_condition)
         self.assertRaises(TypeError, Dependency, 'foo', unknownattribute=42)
 
         d = {}
@@ -67,9 +95,11 @@ class PackageTest(unittest.TestCase):
                           version_lte=2,
                           version_eq=3,
                           version_gte=4,
-                          version_gt=5)
+                          version_gt=5,
+                          condition='$foo == 23 and $bar != 42')
+        dep2.evaluate_condition({'foo': 23, 'bar': 43})
         d[dep2] = None
-        self.assertEquals(len(d), 1)
+        self.assertEqual(len(d), 1)
         dep3 = Dependency('foo',
                           version_lt=1,
                           version_lte=2,
@@ -77,28 +107,33 @@ class PackageTest(unittest.TestCase):
                           version_gte=4,
                           version_gt=6)
         d[dep3] = None
-        self.assertEquals(len(d), 2)
+        self.assertEqual(len(d), 2)
 
+        dep = Dependency('foo', condition='foo > bar and bar < baz')
+        self.assertTrue(dep.evaluate_condition({}))
+
+        dep = Dependency('foo', condition='foo <= bar or bar >= baz')
+        self.assertFalse(dep.evaluate_condition({}))
 
     def test_init_kwargs_string(self):
         pack = Package('foo',
                        name='bar',
                        package_format='1',
-                       version='0.0.0',
-                       version_abi='pabi',
+                       version='0.0.1',
+                       version_compatibility='0.0.0',
                        description='pdesc',
                        licenses=['BSD'],
                        maintainers=[self.get_maintainer()])
         self.assertEqual('foo', pack.filename)
         self.assertEqual('bar', pack.name)
         self.assertEqual('1', pack.package_format)
-        self.assertEqual('pabi', pack.version_abi)
-        self.assertEqual('0.0.0', pack.version)
+        self.assertEqual('0.0.0', pack.version_compatibility)
+        self.assertEqual('0.0.1', pack.version)
         self.assertEqual('pdesc', pack.description)
 
     def test_init_kwargs_object(self):
         mmain = [self.get_maintainer(), self.get_maintainer()]
-        mlis = ['MIT', 'BSD']
+        mlis = ['MIT', License('BSD', 'LICENSE')]
         mauth = [self.get_maintainer(), self.get_maintainer()]
         murl = [Mock(), Mock()]
         mbuilddep = [Mock(), Mock()]
@@ -108,7 +143,12 @@ class PackageTest(unittest.TestCase):
         mconf = [Mock(), Mock()]
         mrepl = [Mock(), Mock()]
         mexp = [Mock(), Mock()]
-        pack = Package(name='bar',
+        mgroup = [
+            self.get_group_dependency('group1'),
+            self.get_group_dependency('group2')]
+        mmember = ['member1', 'member2']
+        pack = Package(package_format='3',
+                       name='bar',
                        version='0.0.0',
                        maintainers=mmain,
                        licenses=mlis,
@@ -120,9 +160,12 @@ class PackageTest(unittest.TestCase):
                        test_depends=mtestdep,
                        conflicts=mconf,
                        replaces=mrepl,
+                       group_depends=mgroup,
+                       member_of_groups=mmember,
                        exports=mexp)
         self.assertEqual(mmain, pack.maintainers)
         self.assertEqual(mlis, pack.licenses)
+        self.assertEqual([None, 'LICENSE'], [l.file for l in pack.licenses])
         self.assertEqual(murl, pack.urls)
         self.assertEqual(mauth, pack.authors)
         self.assertEqual(mbuilddep, pack.build_depends)
@@ -134,14 +177,15 @@ class PackageTest(unittest.TestCase):
         self.assertEqual(mconf, pack.conflicts)
         self.assertEqual(mrepl, pack.replaces)
         self.assertEqual(mexp, pack.exports)
+        self.assertEqual(mgroup, pack.group_depends)
+        self.assertEqual(mmember, pack.member_of_groups)
 
     def test_validate_package(self):
         maint = self.get_maintainer()
         pack = Package('foo',
                        name='bar_2go',
                        package_format='1',
-                       version='0.0.0',
-                       version_abi='pabi',
+                       version='0.0.1',
                        description='pdesc',
                        licenses=['BSD'],
                        maintainers=[maint])
@@ -162,7 +206,7 @@ class PackageTest(unittest.TestCase):
         pack.name = 'bar-bza'
         warnings = []
         pack.validate(warnings=warnings)
-        self.assertIn('naming conventions', warnings[0])
+        self.assertEqual(warnings, [])
 
         pack.name = 'BAR'
         warnings = []
@@ -174,7 +218,7 @@ class PackageTest(unittest.TestCase):
         pack.name = 'bar-bza'
         warnings = []
         pack.validate(warnings=warnings)
-        self.assertEquals(warnings, [])
+        self.assertEqual(warnings, [])
         pack.exports.pop()
 
         # check authors emails
@@ -202,6 +246,18 @@ class PackageTest(unittest.TestCase):
             self.assertRaises(InvalidPackage, Package.validate, pack)
             dep_type.remove(depend)
 
+    def test_invalid_package_exception(self):
+        try:
+            raise InvalidPackage('foo')
+        except InvalidPackage as e:
+            self.assertEqual('foo', str(e))
+            self.assertEqual(None, e.package_path)
+        try:
+            raise InvalidPackage('foo', package_path='./bar')
+        except InvalidPackage as e:
+            self.assertEqual("Error(s) in package './bar':\nfoo", str(e))
+            self.assertEqual('./bar', e.package_path)
+
     def test_validate_person(self):
         auth1 = Person('foo')
         auth1.email = 'foo@bar.com'
@@ -221,20 +277,20 @@ class PackageTest(unittest.TestCase):
     def test_check_known_attributes(self):
 
         def create_node(tag, attrs):
-            data = '<%s %s/>' % (tag, ' '.join(('%s="%s"'%p) for p in attrs.items()))
+            data = '<%s %s/>' % (tag, ' '.join(('%s="%s"' % p) for p in attrs.items()))
             return dom.parseString(data).firstChild
 
         try:
             create_node('tag', {'key': 'value'})
         except Exception as e:
-            self.fail('create_node() raised %s "%s" unexpectedly!' % (type(e),str(e)))
+            self.fail('create_node() raised %s "%s" unexpectedly!' % (type(e), str(e)))
 
-        self.assertRaisesRegexp(Exception, "unbound prefix: line 1, column 0", create_node,'tag', {'ns:key': 'value'})
+        self.assertRaisesRegexp(Exception, 'unbound prefix: line 1, column 0', create_node, 'tag', {'ns:key': 'value'})
 
         try:
             create_node('tag', {'ns:key': 'value', 'xmlns:ns': 'urn:ns'})
         except Exception as e:
-            self.fail('create_node() raised %s "%s" unexpectedly!' % (type(e),str(e)))
+            self.fail('create_node() raised %s "%s" unexpectedly!' % (type(e), str(e)))
 
         def check(attrs, known, res=[]):
             self.assertEqual(res, _check_known_attributes(create_node('tag', attrs), known))
@@ -250,3 +306,18 @@ class PackageTest(unittest.TestCase):
         check({'xmlns:ns': 'urn:ns'}, ['attr'])
         check({'xmlns:ns': 'urn:ns', 'ns:attr': 'value'}, ['attr'])
         check({'xmlns:ns': 'urn:ns', 'ns:attr': 'value', 'attr2': 'value'}, ['attr'], expected_err)
+
+    def test_parse_package_valid(self):
+        filename = os.path.join(test_data_dir, 'valid_package.xml')
+        package = parse_package(filename)
+        assert package.filename == filename
+        assert not package.is_metapackage()
+        assert package.name == 'valid_package'
+        assert package.description == 'valid_package description'
+        assert package.version == '0.1.0'
+        assert package.licenses == ['BSD']
+        assert [x.name for x in package.run_depends] == ['foo', 'bar', 'baz']
+
+    def test_parse_package_invalid(self):
+        filename = os.path.join(test_data_dir, 'invalid_package.xml')
+        self.assertRaises(InvalidPackage, parse_package, filename)
